//--------------------------------------------------------------------------------------------------
// shader to render units                                                                         //
// author: Christian Oberholzer                                                                   //
//--------------------------------------------------------------------------------------------------




//--------------------------------------------------------------------------------------------------
sampler diffuseSampler;
sampler fogOfWarSampler;




//--------------------------------------------------------------------------------------------------
struct DirectionalLight
{
  float3 direction;
  float4 color;
};

struct Material
{
  float4 ambientColor;
  float4 diffuseColor;
  //float4 specularColor;
  //float4 emissiveColor;
};




//--------------------------------------------------------------------------------------------------
void vs_main(
  float4 inObjectPosition : POSITION,
  float3 inObjectNormal : NORMAL,
  float2 inTexCoord : TEXCOORD0,
  float4 inBoneWeights : BLENDWEIGHT,
  float4 inBoneIndices : BLENDINDICES,
  out float4 outPosition : POSITION,
  out float3 outNormal : TEXCOORD2,
  out float2 outTexCoord : TEXCOORD0,
  out float2 outFogOfWarTexCoord : TEXCOORD1,
  uniform float4x4 matWorldArray[26],
  uniform float4x4 matInvTransposeWorldArray[26],
  uniform float4x4 matWorld,
  uniform float4x4 matInvTransposeWorld,
  uniform float4x4 matView,
  uniform float4x4 matProjection,
  uniform float2 mapSize
)
{
  // calculate the pos/normal using the "normal" weights 
  // and accumulate the weights to calculate the last weight
  float3 skinnedPos = float3(0, 0, 0);
  float3 skinnedNormal = float3(0, 0, 0);
  for (int bone = 0; bone < 2; ++bone)
  {
    skinnedPos += mul(matWorldArray[inBoneIndices[bone]], inObjectPosition) * inBoneWeights[bone];
    skinnedNormal += mul(matInvTransposeWorldArray[inBoneIndices[bone]], inObjectNormal) * inBoneWeights[bone];
  }

  float4 worldPosition = mul(matWorld, float4(skinnedPos, 1));
  float4 viewPosition = mul(matView, worldPosition);
  float4 screenPosition = mul(matProjection, viewPosition);
  float4 worldNormal = mul(matInvTransposeWorld, skinnedNormal);

  outPosition = screenPosition;
  outTexCoord = inTexCoord;
  outNormal = worldNormal;
  outFogOfWarTexCoord = float2(worldPosition.x / mapSize.x, -worldPosition.z / mapSize.y);
}




//--------------------------------------------------------------------------------------------------
void ps_main(
  float3 inNormal : TEXCOORD2,
  float2 inTexCoord : TEXCOORD0,
  float2 inFogOfWarTexCoord : TEXCOORD1,
  out float4 outColor : COLOR,
  uniform Material material,
  uniform float4 ambientLight,
  uniform DirectionalLight light,
  uniform float4 teamColor
)
{
  // fetch building texture
  float4 texColor = tex2D(diffuseSampler, inTexCoord);
  float4 fogOfWar = tex2D(fogOfWarSampler, inFogOfWarTexCoord);
  
  // calculate weighting of tex/material-color and team-color
  float teamColorWeight = texColor.a;
  float materialWeight = 1.0 - teamColorWeight;

  // calculate unlighted colors
  float4 ambientColor = float4((material.ambientColor * texColor).rgb * materialWeight + teamColor.rgb * teamColorWeight, 1.0);
  float4 diffuseColor = float4((material.diffuseColor * texColor).rgb * materialWeight + teamColor.rgb * teamColorWeight, 1.0);

  // calculate lighting 
  inNormal = normalize(inNormal);
  float3 toLight = normalize(-light.direction);
  float lambertTerm = saturate(dot(inNormal, toLight));
  float4 diffuseLight = light.color * lambertTerm;

  // combine light-colors and unlighted colors  
  float4 lighted = (
    ambientColor * ambientLight +
    diffuseColor * diffuseLight
  );
  
  // apply fog of war
  outColor = lighted * fogOfWar;
}
