//--------------------------------------------------------------------------------------------------
// shader to render the terrain                                                                   //
// author: Christian Oberholzer                                                                   //
//--------------------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------------------
sampler alphaSampler;
sampler diffuseSampler1;
sampler diffuseSampler2;
sampler diffuseSampler3;
sampler lightMapSampler;
sampler fogOfWarSampler;




//--------------------------------------------------------------------------------------------------
struct DirectionalLight
{
  float3 direction;
  float4 color;
};




//--------------------------------------------------------------------------------------------------
/*
	this first shader pair is used to render the terrain
*/
void vs_main(
  float3 inPosition : POSITION,
  float3 inNormal : NORMAL,
  float2 inAlphaCoord : TEXCOORD0,
  float2 inDiffuseCoord : TEXCOORD1,
  out float4 outPosition : POSITION,
  out float3 outNormal : TEXCOORD2,
  out float2 outAlphaCoord : TEXCOORD0,
  out float2 outDiffuseCoord : TEXCOORD1,
  uniform float4x4 matWorld,
  uniform float4x4 matView,
  uniform float4x4 matProjection
)
{
  outPosition = mul(matWorld, float4(inPosition, 1));
  outPosition = mul(matView, outPosition);
  outPosition = mul(matProjection, outPosition);
  outNormal = mul(matWorld, inNormal);
  outAlphaCoord = inAlphaCoord;
  outDiffuseCoord = inDiffuseCoord;
}

void ps_main(
  float2 inAlphaCoord : TEXCOORD0,
  float2 inDiffuseCoord : TEXCOORD1,
  float3 inNormal : TEXCOORD2,
  out float4 outColor : COLOR,
  uniform DirectionalLight light
)
{
  float4 ambientColor = float4(0.05, 0.05, 0.05, 0.05);
  float4 alpha = tex2D(alphaSampler, inAlphaCoord);
  float scale = 1 / (alpha.a + alpha.r + alpha.g);
  float4 diffuse1 = tex2D(diffuseSampler1, inDiffuseCoord);
  float4 diffuse2 = tex2D(diffuseSampler2, inDiffuseCoord);
  float4 diffuse3 = tex2D(diffuseSampler3, inDiffuseCoord);
  float4 fow = tex2D(fogOfWarSampler, inAlphaCoord);
  float4 lightmap = tex2D(lightMapSampler, inAlphaCoord);
  float4 materialColor =
    diffuse1 * alpha.a * scale +
    diffuse2 * alpha.r * scale +
    diffuse3 * alpha.g * scale
  ;
  inNormal = normalize(inNormal);
  float3 toLight = normalize(-light.direction);
  float lambertTerm = saturate(dot(inNormal, toLight));
  outColor = materialColor * (
    ambientColor +
    light.color * lambertTerm
  ) * fow * lightmap;
}




//--------------------------------------------------------------------------------------------------
/*
	this second shader pair is used to render the landscape preview inside the minimap.
	the only difference to the first shader pair is that we're not using the fog
	of war texture
*/
void landscape_vs_main(
  float3 inPosition : POSITION,
  float3 inNormal : NORMAL,
  float2 inAlphaCoord : TEXCOORD0,
  float2 inDiffuseCoord : TEXCOORD1,
  out float4 outPosition : POSITION,
  out float3 outNormal : TEXCOORD2,
  out float2 outAlphaCoord : TEXCOORD0,
  out float2 outDiffuseCoord : TEXCOORD1,
  uniform float4x4 matWorld,
  uniform float4x4 matView,
  uniform float4x4 matProjection
)
{
  outPosition = mul(matWorld, float4(inPosition, 1));
  outPosition = mul(matView, outPosition);
  outPosition = mul(matProjection, outPosition);
  outNormal = mul(matWorld, inNormal);
  outAlphaCoord = inAlphaCoord;
  outDiffuseCoord = inDiffuseCoord;
}

void landscape_ps_main(
  float2 inAlphaCoord : TEXCOORD0,
  float2 inDiffuseCoord : TEXCOORD1,
  float3 inNormal : TEXCOORD2,
  out float4 outColor : COLOR,
  uniform DirectionalLight light
)
{
  float4 ambientColor = float4(0.15, 0.15, 0.15, 0.15);
  float4 alpha = tex2D(alphaSampler, inAlphaCoord);
  float scale = 1 / (alpha.a + alpha.r + alpha.g);
  float4 diffuse1 = tex2D(diffuseSampler1, inDiffuseCoord);
  float4 diffuse2 = tex2D(diffuseSampler2, inDiffuseCoord);
  float4 diffuse3 = tex2D(diffuseSampler3, inDiffuseCoord);
  float4 lightmap = tex2D(lightMapSampler, inAlphaCoord);
  float4 materialColor =
    diffuse1 * alpha.a * scale +
    diffuse2 * alpha.r * scale +
    diffuse3 * alpha.g * scale
  ;
  inNormal = normalize(inNormal);
  float3 toLight = normalize(-light.direction);
  float lambertTerm = saturate(dot(inNormal, toLight));
  outColor = materialColor * (
    ambientColor +
    light.color * lambertTerm
  ) * lightmap;
}
